﻿Public Class Form1
    Private bPainting As Boolean = False
    Private sTrainingSetsFile As String = "trainingSets.dat"
    ''' <summary>
    ''' Start painting on mouse down.
    ''' </summary>
    Private Sub picOriginalData_MouseDown(sender As Object, e As MouseEventArgs) Handles picOriginalData.MouseDown
        bPainting = True
    End Sub
    ''' <summary>
    ''' Stops painting on mouse leaving the picture box and recreates simplified representation.
    ''' </summary>
    Private Sub picOriginalData_MouseLeave(sender As Object, e As EventArgs) Handles picOriginalData.MouseLeave
        bPainting = False
        calculateSimpleRepresentation()
    End Sub
    ''' <summary>
    ''' Stops painting on mouse up and recreates simplified representation.
    ''' </summary>
    Private Sub picOriginalData_MouseUp(sender As Object, e As MouseEventArgs) Handles picOriginalData.MouseUp
        bPainting = False
        calculateSimpleRepresentation()
    End Sub
    ''' <summary>
    ''' Saving current training sets on closing.
    ''' </summary>
    Private Sub Form1_FormClosing(sender As Object, e As FormClosingEventArgs) Handles Me.FormClosing
        BasicOCR.saveTrainingSets(sTrainingSetsFile)
    End Sub
    ''' <summary>
    ''' Creates a new bitmap to paint on.
    ''' </summary>
    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        clearPaintBitmap()
        If My.Computer.FileSystem.FileExists(sTrainingSetsFile) Then
            Dim iTrainingSetsLoaded As Integer = BasicOCR.loadTrainingSets(sTrainingSetsFile)
            txtTrainingListCount.Text = iTrainingSetsLoaded.ToString
            If iTrainingSetsLoaded > 0 Then
                btnRunTraining.Enabled = True
            End If
        End If
    End Sub
    ''' <summary>
    ''' Moving while painting: Paint the pixel
    ''' </summary>
    Private Sub picOriginalData_MouseMove(sender As Object, e As MouseEventArgs) Handles picOriginalData.MouseMove
        If bPainting = True Then
            If e.X >= 0 And e.Y >= 0 And e.X < picOriginalData.Width And e.Y < picOriginalData.Height Then
                Dim bmp As Bitmap = CType(picOriginalData.Image, Bitmap)
                bmp.SetPixel(e.X, e.Y, Color.Black)
                picOriginalData.Image = bmp
            End If
        End If
    End Sub
    ''' <summary>
    ''' Calculates the binary representation of painted Bitmap and shows it in the second picture box.
    ''' </summary>
    Private Sub calculateSimpleRepresentation()
        Dim bRepresentation(,) As Boolean = Simplifier.getBinaryTiles(CType(picOriginalData.Image, Bitmap), 5, 5, 1)
        picSimpleRepr.Image = TilePainter.writeRepresentationToBitmap(bRepresentation, 40, 40)
    End Sub
    ''' <summary>
    ''' Creates a fresh painting environment.
    ''' </summary>
    Private Sub clearPaintBitmap()
        Dim bmp As Bitmap = New Bitmap(picOriginalData.Width, picOriginalData.Height)
        picOriginalData.Image = bmp
        Dim g As Graphics = Graphics.FromImage(picOriginalData.Image)
        g.Clear(Color.White)
    End Sub
    ''' <summary>
    ''' Creates a fresh painting environment.
    ''' </summary>
    Private Sub btnClear_Click(sender As Object, e As EventArgs) Handles btnClear.Click
        clearPaintBitmap()
        calculateSimpleRepresentation()
    End Sub
    ''' <summary>
    ''' Does a linear training of neural net in order to recognice painted letter.
    ''' </summary>
    Private Sub btnTrain_Click(sender As Object, e As EventArgs) Handles btnAddToTrainingList.Click
        btnAddToTrainingList.Enabled = False
        Application.DoEvents()
        Dim bRepr As Boolean(,) = Simplifier.getBinaryTiles(CType(picOriginalData.Image, Bitmap), 5, 5, 1)
        txtTrainingListCount.Text = BasicOCR.addTrainingSet(bRepr, txtChar.Text(0)).ToString
        btnAddToTrainingList.Enabled = True
        btnRunTraining.Enabled = True
    End Sub
    ''' <summary>
    ''' Converts painted image into simplified binary representation and asks neural net which letter has been painted.
    ''' </summary>
    Private Sub btnRecognize_Click(sender As Object, e As EventArgs) Handles btnRecognize.Click
        Dim bRepr As Boolean(,) = Simplifier.getBinaryTiles(CType(picOriginalData.Image, Bitmap), 5, 5, 1)
        Dim rr As BasicOCR.RecognitionResult = BasicOCR.getResult(bRepr)
        MsgBox("Recognized letter: " & rr.cLetter.ToString & vbNewLine & vbNewLine & "Brain is to " & Math.Round(rr.dQuality * 100, 1).ToString & "% sure.", MsgBoxStyle.Information)
    End Sub
    ''' <summary>
    ''' Runs network training for all training sets.
    ''' </summary>
    Private Sub btnRunTraining_Click(sender As Object, e As EventArgs) Handles btnRunTraining.Click
        btnRunTraining.Enabled = False
        Application.DoEvents()
        If rbnLinear.Checked = True Then
            BasicOCR.trainTrainingSet(0.2, BasicOCR.TrainingMode.Linear)
        Else
            BasicOCR.trainTrainingSet(0.2, BasicOCR.TrainingMode.Randomly)
        End If
        btnRunTraining.Enabled = True
    End Sub
End Class
